.text
.align 2

.global cycleTimes
cycleTimes:
.word 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

.global testCount
testCount:
.word 0



.global SpeedTest
.arm
.align 2
SpeedTest:
stmfd	sp!, {r4-r11, lr}

mov		r0, sp					@ Going to move DTCM, so stack needs
mov		sp, #0x2300000			@ to go out into main RAM somewhere.
stmfd	sp!, {r0}				@ Save the old sp.


@ - Set up the TCM -

ldr		r0, =0x04000208			@ IME
mov		r1, #0
strh	r1, [r0]				@ Disable interrupts

	@ Back up the old ones
mrc		p15, 0, r0, c1, c0, 0	@ TCM/protection unit control
mrc		p15, 0, r1, c9, c1, 1	@ ITCM
mrc		p15, 0, r2, c9, c1, 0	@ DTCM
adr		r3, tcmRegs
stmia	r3, {r0-r2}

	@ Set them up for the tests
ldr		r1, =0x00000078			@ disable TCM and protection unit
mcr		p15, 0, r1, c1, c0, 0

ldr		r0, =0x00000020
mcr		p15, 0, r0, c9, c1, 1	@ ITCM base = 0x00000*4096, size = 32MB (Mirror to 0x2000000)

ldr		r0, =0x02FFC00A
mcr		p15, 0, r0, c9, c1, 0	@ DTCM base = 0x02FFC*4096, size = 16KB, easy offset from 0x3000000

ldr		r1, =0x00052078			@ enable TCM
mcr		p15, 0, r1, c1, c0, 0


@ - Copy the tester to ITCM -

ldr		r0, =0x00000000			@ ITCM base
ldr		r1, =testsInITCM
ldr		r2, =testsInITCM_end
sub		r2, r2, r1

copyTesterLoop:
ldr		r3, [r1], #4
str		r3, [r0], #4
subs	r2, r2, #4
bgt		copyTesterLoop

ldr		r0, =0x00000000
mov		lr, pc
bx		r0


@ - Restore the old TCM settings -

adr		r3, tcmRegs
ldmia	r3, {r0-r2}

ldr		r1, =0x00000078			@ disable TCM and protection unit
mcr		p15, 0, r1, c1, c0, 0

mcr		p15, 0, r1, c9, c1, 1	@ ITCM
mcr		p15, 0, r2, c9, c1, 0	@ DTCM
mcr		p15, 0, r0, c1, c0, 0	@ TCM/protection unit control


@ - Return -

ldmfd	sp!, {r0}				@ Load the original sp
mov		sp, r0

ldr		r0, =0x04000208
mov		r1, #1
strh	r1, [r0]

ldmfd	sp!, {r4-r11, lr}
bx		lr

.pool

tcmRegs:
.word 0,0,0




@ ----- Everything from here on is copied to ITCM -----

testsInITCM:
stmfd	sp!, {lr}

adr		r0, dataPool
ldmia	r0, {r4, r5}	@ r4=cycleTimes, r5=&testCount
mov		r6, #0			@ r6=testCount


@ - Run tests (CHANGEME) -

adr		r0, TestLoop
bl		runTest
adr		r0, TestLdr
bl		runTest
adr		r0, TestLdrLock
bl		runTest
adr		r0, TestLdrMainRam
bl		runTest
adr		r0, TestLdrLdr
bl		runTest
adr		r0, TestLdrLdrMainRam
bl		runTest
adr		r0, TestQadd
bl		runTest
adr		r0, TestSmulxyQadd
bl		runTest
adr		r0, TestLdm
bl		runTest
adr		r0, TestLdmMainRam
bl		runTest

@ - End of tests -


str		r6, [r5]

ldmfd	sp!, {lr}
bx		lr



	@ Args: r0=address of test function
runTest:
stmfd	sp!, {lr}
mov		r1, #0x04000000
mov		r2, #0x00800000		@ Timer enable, count 0
str		r2, [r1, #0x100]	@ Timer 0 count and control

mov		lr, pc
bx		r0

mov		r2, #0x04000000
add		r1, r2, #0x100		@ Timer 0 offset
ldrh	r0, [r1]			@ Read timer 0 count
strh	r2, [r1, #2]		@ Disable timer (lower 16 bits of r2 are 0)

mov		r0, r0, lsl #1		@ Timers are 33MHz, convert to actual cycles
str		r0, [r4], #4		@ Store to cycleCounts
add		r6, r6, #1			@ Increment testCount

ldmfd	sp!, {lr}
bx		lr



dataPool:
.word cycleTimes
.word testCount

.pool



@ ----- Test functions (CHANGEME) -----

TestLoop:
mov		r0, #1024
TestLoop_loop:
subs	r0, r0, #1
bne		TestLoop_loop
bx		lr


TestLdr:
mov		r0, #1024
mov		r1, #0x3000000
sub		r1, r1, #0x4000		@ r1=DTCM
TestLdr_loop:
ldr		r2, [r1]
subs	r0, r0, #1
bne		TestLdr_loop
bx		lr


TestLdrLock:
mov		r0, #1024
mov		r1, #0x3000000
sub		r1, r1, #0x4000		@ r1=DTCM
TestLdrLock_loop:
ldr		r2, [r1]
add		r2, r2, r2			@ Cause an interlock
subs	r0, r0, #1
bne		TestLdrLock_loop
bx		lr


TestLdrMainRam:
mov		r0, #1024
mov		r1, #0x2000000
TestLdrMainRam_loop:
ldr		r2, [r1]
subs	r0, r0, #1
bne		TestLdrMainRam_loop
bx		lr


TestLdrLdr:
mov		r0, #1024
mov		r1, #0x3000000
sub		r1, r1, #0x4000		@ r1=DTCM
TestLdrLdr_loop:
ldr		r2, [r1]
ldr		r3, [r1, #4]
subs	r0, r0, #1
bne		TestLdrLdr_loop
bx		lr


TestLdrLdrMainRam:
mov		r0, #1024
mov		r1, #0x2000000
TestLdrLdrMainRam_loop:
ldr		r2, [r1]
ldr		r3, [r1, #4]
subs	r0, r0, #1
bne		TestLdrLdrMainRam_loop
bx		lr


TestQadd:
mov		r0, #1024
TestQadd_loop:
.word 0xe1021051
@qadd	r1, r1, r2
subs	r0, r0, #1
bne		TestQadd_loop
bx		lr


TestSmulxyQadd:
mov		r0, #1024
TestSmulxyQadd_loop:
.word 0xe16201c1
.word 0xe1031052
@smulbt	r2, r1, r1
@qadd	r3, r2, r1
subs	r0, r0, #1
bne		TestSmulxyQadd_loop
bx		lr


TestLdm:
stmfd	sp!, {r4-r11}
mov		r0, #1024
mov		r1, #0x3000000
sub		r1, r1, #0x4000		@ r1=DTCM
TestLdm_loop:
ldmia	r1, {r4-r11}
subs	r0, r0, #1
bne		TestLdm_loop
ldmfd	sp!, {r4-r11}
bx		lr


TestLdmMainRam:
stmfd	sp!, {r4-r11}
mov		r0, #1024
mov		r1, #0x2000000
TestLdmMainRam_loop:
ldmia	r1, {r4-r11}
subs	r0, r0, #1
bne		TestLdmMainRam_loop
ldmfd	sp!, {r4-r11}
bx		lr

@ ----- End of test functions -----



testsInITCM_end:
